(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1996 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  under the terms of the GNU Library General Public License, with    *)
(*  the special exception on linking described in file ../LICENSE.     *)
(*                                                                     *)
(***********************************************************************)

(* SHA-256 support (C) Copyright 2006-2007 Peter Selinger *)

(* $Id: sha256.ml,v 1.10 2003/12/31 14:20:39 doligez Exp $ *)

(* Message digest (SHA-256) *)

type t = string

type state

external sha256_new: unit -> state = "caml_sha256_new"
external sha256_data: state -> string -> int -> int -> unit = "caml_sha256_data"
external sha256_finish: state -> string = "caml_sha256_finish"

let digest_length = 32

let string str =
  let st = sha256_new() in
  sha256_data st str 0 (String.length str);
  sha256_finish st
    
let substring str ofs len =
  if ofs < 0 || len < 0 || ofs > String.length str - len
  then invalid_arg "Digest.substring"
  else 
    let st = sha256_new() in
    sha256_data st str ofs len;
    sha256_finish st

let channel ic =
  let st = sha256_new() in
  let bufsize = 409600 in
  let buf = String.create bufsize in
  let rec hash() =
    let r = input ic buf 0 bufsize in
    if r = 0 then 
      sha256_finish st 
    else begin
      sha256_data st buf 0 r;
      hash()
    end
  in
  hash()

let file filename =
  let ic = open_in_bin filename in
  let d = channel ic in
  close_in ic;
  d

let output chan digest =
  output chan digest 0 digest_length

let input chan =
  let digest = String.create digest_length in
  really_input chan digest 0 digest_length;
  digest

let to_hex d =
  let result = String.create (digest_length*2) in
  for i = 0 to (digest_length-1) do
    String.blit (Printf.sprintf "%02x" (int_of_char d.[i])) 0 result (2*i) 2;
  done;
  result
;;
